@@ -2,8 +2,10 @@
 //! It comprises a variety of basic data types, such as the DICOM attribute tag, the
 //! element header, and element composite types.
 
-use crate::value::{CastValueError, ConvertValueError, PrimitiveValue, Value};
-use chrono::{DateTime, FixedOffset, NaiveDate, NaiveTime};
+use crate::value::{
+    CastValueError, ConvertValueError, DicomDate, DicomDateTime, DicomTime, PrimitiveValue, Value,
+};
+use chrono::FixedOffset;
 use num_traits::NumCast;
 use snafu::{Backtrace, Snafu};
 use std::borrow::Cow;
@@ -232,7 +234,13 @@ impl<I, P> DataElement<I, P>
 where
     I: HasLength,
 {
-    /// Create a primitive data element from the given parts.
+    /// Create a primitive data element from the given parts,
+    /// where the length is inferred from the value's byte length.
+    ///
+    /// If the value is textual,
+    /// the byte length of that value encoded in UTF-8 is assumed.
+    /// If you already have a length in this context,
+    /// prefer calling `new_with_len` instead.
     ///
     /// This method will not check whether the value representation is
     /// compatible with the given value.
@@ -251,14 +259,49 @@ where
         }
     }
 
-    /// Retrieve the element's value as a single string.
+    /// Create a primitive data element from the given parts.
+    ///
+    /// This method will not check
+    /// whether the length accurately represents the given value's byte length,
+    /// nor whether the value representation is compatible with the value.
+    pub fn new_with_len<T>(tag: Tag, vr: VR, length: Length, value: T) -> Self
+    where
+        T: Into<Value<I, P>>,
+    {
+        let value = value.into();
+        DataElement {
+            header: DataElementHeader {
+                tag,
+                vr,
+                len: length,
+            },
+            value,
+        }
+    }
+
+    /// Retrieve the element's value as a single clean string,
+    /// with no trailing whitespace.
+    ///
+    /// Returns an error if the value is not primitive.
     pub fn to_str(&self) -> Result<Cow<str>, CastValueError> {
         self.value.to_str()
     }
 
+    /// Retrieve the element's value as a single raw string,
+    /// with trailing whitespace kept.
+    ///
+    /// Returns an error if the value is not primitive.
+    pub fn to_raw_str(&self) -> Result<Cow<str>, CastValueError> {
+        self.value.to_raw_str()
+    }
+
     /// Retrieves the element's value as a clean string
+    #[deprecated(
+        note = "`to_clean_str()` is now deprecated in favour of using `to_str()` directly. 
+        `to_raw_str()` replaces the old functionality of `to_str()` and maintains all trailing whitespace."
+    )]
     pub fn to_clean_str(&self) -> Result<Cow<str>, CastValueError> {
-        self.value.to_clean_str()
+        self.value.to_str()
     }
 
     /// Convert the full primitive value into raw bytes.
@@ -372,78 +415,72 @@ where
     /// Retrieve and convert the primitive value into a date.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a `NaiveDate` as described in [`PrimitiveValue::to_date`].
+    /// a `DicomDate` as described in [`PrimitiveValue::to_date`].
     ///
     /// Returns an error if the value is not primitive.
     ///
-    /// [`PrimitiveValue::to_date`]: ../enum.PrimitiveValue.html#to_date
-    pub fn to_date(&self) -> Result<NaiveDate, ConvertValueError> {
+    pub fn to_date(&self) -> Result<DicomDate, ConvertValueError> {
         self.value().to_date()
     }
 
     /// Retrieve and convert the primitive value into a sequence of dates.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a vector of `NaiveDate` as described in [`PrimitiveValue::to_multi_date`].
+    /// a vector of `DicomDate` as described in [`PrimitiveValue::to_multi_date`].
     ///
     /// Returns an error if the value is not primitive.
     ///
-    /// [`PrimitiveValue::to_multi_date`]: ../enum.PrimitiveValue.html#to_multi_date
-    pub fn to_multi_date(&self) -> Result<Vec<NaiveDate>, ConvertValueError> {
+    pub fn to_multi_date(&self) -> Result<Vec<DicomDate>, ConvertValueError> {
         self.value().to_multi_date()
     }
 
     /// Retrieve and convert the primitive value into a time.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a `NaiveTime` as described in [`PrimitiveValue::to_time`].
+    /// a `DicomTime` as described in [`PrimitiveValue::to_time`].
     ///
     /// Returns an error if the value is not primitive.
     ///
-    /// [`PrimitiveValue::to_time`]: ../enum.PrimitiveValue.html#to_time
-    pub fn to_time(&self) -> Result<NaiveTime, ConvertValueError> {
+    pub fn to_time(&self) -> Result<DicomTime, ConvertValueError> {
         self.value().to_time()
     }
 
     /// Retrieve and convert the primitive value into a sequence of times.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a vector of `NaiveTime` as described in [`PrimitiveValue::to_multi_time`].
+    /// a vector of `DicomTime` as described in [`PrimitiveValue::to_multi_time`].
     ///
     /// Returns an error if the value is not primitive.
     ///
-    /// [`PrimitiveValue::to_multi_time`]: ../enum.PrimitiveValue.html#to_multi_time
-    pub fn to_multi_time(&self) -> Result<Vec<NaiveTime>, ConvertValueError> {
+    pub fn to_multi_time(&self) -> Result<Vec<DicomTime>, ConvertValueError> {
         self.value().to_multi_time()
     }
 
     /// Retrieve and convert the primitive value into a date-time.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a `DateTime` as described in [`PrimitiveValue::to_datetime`].
+    /// a `DicomDateTime` as described in [`PrimitiveValue::to_datetime`].
     ///
     /// Returns an error if the value is not primitive.
     ///
-    /// [`PrimitiveValue::to_datetime`]: ../enum.PrimitiveValue.html#to_datetime
     pub fn to_datetime(
         &self,
         default_offset: FixedOffset,
-    ) -> Result<DateTime<FixedOffset>, ConvertValueError> {
+    ) -> Result<DicomDateTime, ConvertValueError> {
         self.value().to_datetime(default_offset)
     }
 
     /// Retrieve and convert the primitive value into a sequence of date-times.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a vector of `DateTime` as described in [`PrimitiveValue::to_multi_datetime`].
+    /// a vector of `DicomDateTime` as described in [`PrimitiveValue::to_multi_datetime`].
     ///
     /// Returns an error if the value is not primitive.
     ///
-    /// [`PrimitiveValue::to_multi_datetime`]: ../enum.PrimitiveValue.html#to_multi_datetime
     pub fn to_multi_datetime(
         &self,
         default_offset: FixedOffset,
-    ) -> Result<Vec<DateTime<FixedOffset>>, ConvertValueError> {
+    ) -> Result<Vec<DicomDateTime>, ConvertValueError> {
         self.value().to_multi_datetime(default_offset)
     }
 }
@@ -473,7 +510,7 @@ where
 
     /// Retrieves the DICOM value.
     pub fn value(&self) -> &Value<I, P> {
-        &self.value
+        self.value
     }
 }
 
@@ -530,9 +567,9 @@ impl<I, P> DataElement<I, P> {
         self.value().strings()
     }
 
-    impl_primitive_getters!(date, dates, Date, NaiveDate);
-    impl_primitive_getters!(time, times, Time, NaiveTime);
-    impl_primitive_getters!(datetime, datetimes, DateTime, DateTime<FixedOffset>);
+    impl_primitive_getters!(date, dates, Date, DicomDate);
+    impl_primitive_getters!(time, times, Time, DicomTime);
+    impl_primitive_getters!(datetime, datetimes, DateTime, DicomDateTime);
     impl_primitive_getters!(uint8, uint8_slice, U8, u8);
     impl_primitive_getters!(uint16, uint16_slice, U16, u16);
     impl_primitive_getters!(int16, int16_slice, I16, i16);
@@ -638,7 +675,7 @@ impl SequenceItemHeader {
                 // item delimiter
                 // delimiters should not have a positive length
                 if len != Length(0) {
-                    UnexpectedDelimiterLength { len }.fail()
+                    UnexpectedDelimiterLengthSnafu { len }.fail()
                 } else {
                     Ok(SequenceItemHeader::ItemDelimiter)
                 }
@@ -647,7 +684,7 @@ impl SequenceItemHeader {
                 // sequence delimiter
                 Ok(SequenceItemHeader::SequenceDelimiter)
             }
-            tag => UnexpectedTag { tag }.fail(),
+            tag => UnexpectedTagSnafu { tag }.fail(),
         }
     }
 }
@@ -1161,7 +1198,7 @@ mod tests {
         let p = dicom_value!(U16, [256, 0, 16]);
         let val = DicomValue::new(p);
         let element = DataElement::new(Tag(0x0028, 0x3002), VR::US, val);
-        assert_eq!(element.to_clean_str().unwrap(), "256\\0\\16",);
+        assert_eq!(element.to_str().unwrap(), "256\\0\\16",);
     }
 
     #[test]
@@ -1188,7 +1225,7 @@ mod tests {
 
         assert_eq!(
             data_element.to_date().unwrap(),
-            NaiveDate::from_ymd(1994, 10, 12),
+            DicomDate::from_ymd(1994, 10, 12).unwrap(),
         );
     }
 
