@@ -6,16 +6,21 @@ use smallvec::SmallVec;
 use std::{borrow::Cow, str::FromStr};
 
 pub mod deserialize;
+pub mod partial;
 mod primitive;
+pub mod range;
 pub mod serialize;
 
 pub use self::deserialize::Error as DeserializeError;
+pub use self::partial::{DicomDate, DicomDateTime, DicomTime};
+pub use self::range::{AsRange, DateRange, DateTimeRange, TimeRange};
+
 pub use self::primitive::{
     CastValueError, ConvertValueError, InvalidValueReadError, PrimitiveValue, ValueType,
 };
 
 /// re-exported from chrono
-use chrono::{DateTime, FixedOffset, NaiveDate, NaiveTime};
+use chrono::FixedOffset;
 
 /// An aggregation of one or more elements in a value.
 pub type C<T> = SmallVec<[T; 2]>;
@@ -57,7 +62,7 @@ pub enum Value<I = EmptyObject, P = [u8; 0]> {
     /// An encapsulated pixel data sequence.
     PixelSequence {
         /// The value contents of the offset table.
-        offset_table: C<u8>,
+        offset_table: C<u32>,
         /// The sequence of compressed fragments.
         fragments: C<P>,
     },
@@ -69,7 +74,7 @@ impl<P> Value<EmptyObject, P> {
     ///
     /// Note: This function does not validate the offset table
     /// against the fragments.
-    pub fn new_pixel_sequence<T>(offset_table: C<u8>, fragments: T) -> Self
+    pub fn new_pixel_sequence<T>(offset_table: C<u32>, fragments: T) -> Self
     where
         T: Into<C<P>>,
     {
@@ -153,9 +158,9 @@ impl<I, P> Value<I, P> {
     }
 
     /// Gets a reference to the encapsulated pixel data's offset table.
-    pub fn offset_table(&self) -> Option<&[u8]> {
+    pub fn offset_table(&self) -> Option<&[u32]> {
         match self {
-            Value::PixelSequence { offset_table, .. } => Some(&offset_table),
+            Value::PixelSequence { offset_table, .. } => Some(offset_table),
             _ => None,
         }
     }
@@ -193,9 +198,12 @@ impl<I, P> Value<I, P>
 where
     I: HasLength,
 {
-    /// Convert the full primitive value into a single string.
+    /// Convert the full primitive value into a clean string.
     ///
-    /// If the value contains multiple strings, they are concatenated
+    /// The value is converted into a strings
+    /// as described in [`PrimitiveValue::to_str`].
+    /// If the value contains multiple strings,
+    /// they are trimmed at the end and concatenated
     /// (separated by the standard DICOM value delimiter `'\\'`)
     /// into an owned string.
     ///
@@ -210,12 +218,17 @@ where
         }
     }
 
-    /// Convert the full primitive value into a clean string.
+    /// Convert the full primitive value into a single raw string,
+    /// with trailing whitespace kept.
+    ///
+    /// If the value contains multiple strings, they are concatenated
+    /// (separated by the standard DICOM value delimiter `'\\'`)
+    /// into an owned string.
     ///
     /// Returns an error if the value is not primitive.
-    pub fn to_clean_str(&self) -> Result<Cow<str>, CastValueError> {
+    pub fn to_raw_str(&self) -> Result<Cow<str>, CastValueError> {
         match self {
-            Value::Primitive(prim) => Ok(prim.to_clean_str()),
+            Value::Primitive(prim) => Ok(prim.to_raw_str()),
             _ => Err(CastValueError {
                 requested: "string",
                 got: self.value_type(),
@@ -223,6 +236,17 @@ where
         }
     }
 
+    /// Convert the full primitive value into a clean string.
+    ///
+    /// Returns an error if the value is not primitive.
+    #[deprecated(
+        note = "`to_clean_str()` is now deprecated in favour of using `to_str()` directly. 
+        `to_raw_str()` replaces the old functionality of `to_str()` and maintains all trailing whitespace."
+    )]
+    pub fn to_clean_str(&self) -> Result<Cow<str>, CastValueError> {
+        self.to_str()
+    }
+
     /// Convert the full primitive value into a sequence of strings.
     ///
     /// If the value is a primitive, it will be converted into
@@ -257,20 +281,6 @@ where
         }
     }
 
-    /// Retrieves the primitive value as a sequence of unsigned bytes
-    /// without conversions.
-    #[deprecated(note = "use `uint8_slice` instead")]
-    pub fn as_u8(&self) -> Result<&[u8], CastValueError> {
-        self.uint8_slice()
-    }
-
-    /// Retrieves the primitive value as a sequence of signed 32-bit integers
-    /// without conversions.
-    #[deprecated(note = "use `int32_slice` instead")]
-    pub fn as_i32(&self) -> Result<&[i32], CastValueError> {
-        self.int32_slice()
-    }
-
     /// Retrieve and convert the primitive value into an integer.
     ///
     /// If the value is a primitive, it will be converted into
@@ -387,108 +397,153 @@ where
         }
     }
 
-    /// Retrieve and convert the primitive value into a date.
+    /// Retrieve and convert the primitive value into a `DicomDate`.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a `NaiveDate` as described in [`PrimitiveValue::to_date`].
+    /// a `DicomDate` as described in [`PrimitiveValue::to_date`].
     ///
-    /// [`PrimitiveValue::to_date`]: ../enum.PrimitiveValue.html#to_date
-    pub fn to_date(&self) -> Result<NaiveDate, ConvertValueError> {
+    pub fn to_date(&self) -> Result<DicomDate, ConvertValueError> {
         match self {
             Value::Primitive(v) => v.to_date(),
             _ => Err(ConvertValueError {
-                requested: "Date",
+                requested: "DicomDate",
                 original: self.value_type(),
                 cause: None,
             }),
         }
     }
 
-    /// Retrieve and convert the primitive value into a sequence of dates.
+    /// Retrieve and convert the primitive value into a sequence of `DicomDate`s.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a vector of `NaiveDate` as described in [`PrimitiveValue::to_multi_date`].
+    /// a vector of `DicomDate` as described in [`PrimitiveValue::to_multi_date`].
     ///
-    /// [`PrimitiveValue::to_multi_date`]: ../enum.PrimitiveValue.html#to_multi_date
-    pub fn to_multi_date(&self) -> Result<Vec<NaiveDate>, ConvertValueError> {
+    pub fn to_multi_date(&self) -> Result<Vec<DicomDate>, ConvertValueError> {
         match self {
             Value::Primitive(v) => v.to_multi_date(),
             _ => Err(ConvertValueError {
-                requested: "Date",
+                requested: "DicomDate",
                 original: self.value_type(),
                 cause: None,
             }),
         }
     }
 
-    /// Retrieve and convert the primitive value into a time.
+    /// Retrieve and convert the primitive value into a `DicomTime`.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a `NaiveTime` as described in [`PrimitiveValue::to_time`].
+    /// a `DicomTime` as described in [`PrimitiveValue::to_time`].
     ///
-    /// [`PrimitiveValue::to_time`]: ../enum.PrimitiveValue.html#to_time
-    pub fn to_time(&self) -> Result<NaiveTime, ConvertValueError> {
+    pub fn to_time(&self) -> Result<DicomTime, ConvertValueError> {
         match self {
             Value::Primitive(v) => v.to_time(),
             _ => Err(ConvertValueError {
-                requested: "Time",
+                requested: "DicomTime",
                 original: self.value_type(),
                 cause: None,
             }),
         }
     }
 
-    /// Retrieve and convert the primitive value into a sequence of times.
+    /// Retrieve and convert the primitive value into a sequence of `DicomTime`s.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a vector of `NaiveTime` as described in [`PrimitiveValue::to_multi_time`].
+    /// a vector of `DicomTime` as described in [`PrimitiveValue::to_multi_time`].
     ///
-    /// [`PrimitiveValue::to_multi_time`]: ../enum.PrimitiveValue.html#to_multi_time
-    pub fn to_multi_time(&self) -> Result<Vec<NaiveTime>, ConvertValueError> {
+    pub fn to_multi_time(&self) -> Result<Vec<DicomTime>, ConvertValueError> {
         match self {
             Value::Primitive(v) => v.to_multi_time(),
             _ => Err(ConvertValueError {
-                requested: "Time",
+                requested: "DicomTime",
                 original: self.value_type(),
                 cause: None,
             }),
         }
     }
 
-    /// Retrieve and convert the primitive value into a date-time.
+    /// Retrieve and convert the primitive value into a `DicomDateTime`.
     ///
     /// If the value is a primitive, it will be converted into
     /// a `DateTime` as described in [`PrimitiveValue::to_datetime`].
     ///
-    /// [`PrimitiveValue::to_datetime`]: ../enum.PrimitiveValue.html#to_datetime
     pub fn to_datetime(
         &self,
         default_offset: FixedOffset,
-    ) -> Result<DateTime<FixedOffset>, ConvertValueError> {
+    ) -> Result<DicomDateTime, ConvertValueError> {
         match self {
             Value::Primitive(v) => v.to_datetime(default_offset),
             _ => Err(ConvertValueError {
-                requested: "DateTime",
+                requested: "DicomDateTime",
                 original: self.value_type(),
                 cause: None,
             }),
         }
     }
 
-    /// Retrieve and convert the primitive value into a sequence of date-times.
+    /// Retrieve and convert the primitive value into a sequence of `DicomDateTime`s.
     ///
     /// If the value is a primitive, it will be converted into
-    /// a vector of `DateTime` as described in [`PrimitiveValue::to_multi_datetime`].
+    /// a vector of `DicomDateTime` as described in [`PrimitiveValue::to_multi_datetime`].
     ///
-    /// [`PrimitiveValue::to_multi_datetime`]: ../enum.PrimitiveValue.html#to_multi_datetime
     pub fn to_multi_datetime(
         &self,
         default_offset: FixedOffset,
-    ) -> Result<Vec<DateTime<FixedOffset>>, ConvertValueError> {
+    ) -> Result<Vec<DicomDateTime>, ConvertValueError> {
         match self {
             Value::Primitive(v) => v.to_multi_datetime(default_offset),
             _ => Err(ConvertValueError {
-                requested: "DateTime",
+                requested: "DicomDateTime",
+                original: self.value_type(),
+                cause: None,
+            }),
+        }
+    }
+
+    /// Retrieve and convert the primitive value into a `DateRange`.
+    ///
+    /// If the value is a primitive, it will be converted into
+    /// a `DateRange` as described in [`PrimitiveValue::to_date_range`].
+    ///
+    pub fn to_date_range(&self) -> Result<DateRange, ConvertValueError> {
+        match self {
+            Value::Primitive(v) => v.to_date_range(),
+            _ => Err(ConvertValueError {
+                requested: "DateRange",
+                original: self.value_type(),
+                cause: None,
+            }),
+        }
+    }
+
+    /// Retrieve and convert the primitive value into a `TimeRange`.
+    ///
+    /// If the value is a primitive, it will be converted into
+    /// a `TimeRange` as described in [`PrimitiveValue::to_time_range`].
+    ///
+    pub fn to_time_range(&self) -> Result<TimeRange, ConvertValueError> {
+        match self {
+            Value::Primitive(v) => v.to_time_range(),
+            _ => Err(ConvertValueError {
+                requested: "TimeRange",
+                original: self.value_type(),
+                cause: None,
+            }),
+        }
+    }
+
+    /// Retrieve and convert the primitive value into a `DateTimeRange`.
+    ///
+    /// If the value is a primitive, it will be converted into
+    /// a `DateTimeRange` as described in [`PrimitiveValue::to_datetime_range`].
+    ///
+    pub fn to_datetime_range(
+        &self,
+        offset: FixedOffset,
+    ) -> Result<DateTimeRange, ConvertValueError> {
+        match self {
+            Value::Primitive(v) => v.to_datetime_range(offset),
+            _ => Err(ConvertValueError {
+                requested: "DateTimeRange",
                 original: self.value_type(),
                 cause: None,
             }),
@@ -505,12 +560,6 @@ where
             }),
         }
     }
-
-    /// Retrieves the primitive value as a sequence of DICOM tags.
-    #[deprecated(note = "use `tags` instead")]
-    pub fn as_tags(&self) -> Result<&[Tag], CastValueError> {
-        self.tags()
-    }
 }
 
 /// Macro for implementing getters to single and multi-values,
@@ -591,9 +640,9 @@ impl<I, P> Value<I, P> {
     }
 
     impl_primitive_getters!(tag, tags, Tags, Tag);
-    impl_primitive_getters!(date, dates, Date, NaiveDate);
-    impl_primitive_getters!(time, times, Time, NaiveTime);
-    impl_primitive_getters!(datetime, datetimes, DateTime, DateTime<FixedOffset>);
+    impl_primitive_getters!(date, dates, Date, DicomDate);
+    impl_primitive_getters!(time, times, Time, DicomTime);
+    impl_primitive_getters!(datetime, datetimes, DateTime, DicomDateTime);
     impl_primitive_getters!(uint8, uint8_slice, U8, u8);
     impl_primitive_getters!(uint16, uint16_slice, U16, u16);
     impl_primitive_getters!(int16, int16_slice, I16, i16);
@@ -710,27 +759,29 @@ mod tests {
         ));
 
         assert_eq!(
-            Value::new(PrimitiveValue::Date(smallvec![NaiveDate::from_ymd(
+            Value::new(PrimitiveValue::Date(smallvec![DicomDate::from_ymd(
                 2014, 10, 12
-            )]))
+            )
+            .unwrap()]))
             .date()
-            .unwrap(),
-            NaiveDate::from_ymd(2014, 10, 12),
+            .ok(),
+            Some(DicomDate::from_ymd(2014, 10, 12).unwrap()),
         );
 
         assert_eq!(
             Value::new(PrimitiveValue::Date(
-                smallvec![NaiveDate::from_ymd(2014, 10, 12); 5]
+                smallvec![DicomDate::from_ymd(2014, 10, 12).unwrap(); 5]
             ))
             .dates()
             .unwrap(),
-            &[NaiveDate::from_ymd(2014, 10, 12); 5]
+            &[DicomDate::from_ymd(2014, 10, 12).unwrap(); 5]
         );
 
         assert!(matches!(
-            Value::new(PrimitiveValue::Date(smallvec![NaiveDate::from_ymd(
+            Value::new(PrimitiveValue::Date(smallvec![DicomDate::from_ymd(
                 2014, 10, 12
-            )]))
+            )
+            .unwrap()]))
             .time(),
             Err(CastValueError {
                 requested: "time",
